package dev.peerat.parser.java;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.function.Predicate;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class AnnotationClass extends ClassBase{
	
	private Token extend; //TODO verify?
	
	private List<JavaElement> elements;
	
	public AnnotationClass(List<Annotation> annotations, int mod, Token name){
		super(annotations, mod, name);
		this.elements = new ArrayList<>();
	}
	
	@Override
	public void addFunction(Function function) {
		this.elements.add(function);
	}

	@Override
	public void addVariable(Variable variable) {
		this.elements.add(variable);
	}

	@Override
	public void addClass(ClassBase clazz){
		this.elements.add(clazz);
	}
	
	public Token getExtension(){
		return this.extend;
	}
	
	public List<JavaElement> getElements(){
		return this.elements;
	}
	
	@Override
	public <E extends JavaElement> E find(Predicate<JavaElement> finder) {
		E annotation = super.find(finder);
		if(annotation != null) return annotation;
		
		if(elements != null){
			for(JavaElement element : this.elements){
				if(finder.test(element)) return (E) element;
			}
		}
		return null;
	}

	@Override
	public <E extends JavaElement> void findAll(Predicate<JavaElement> finder, Collection<E> collector) {
		super.findAll(finder, collector);
		if(elements != null){
			for(JavaElement element : this.elements){
				if(finder.test(element)) collector.add((E) element);
				element.findAll(finder, collector);
			}
		}
	}

	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		if(!visitor.canPropagate()) return bag;
		
		bag.merge(super.visit(visitor));
		
		for(JavaElement element : this.elements){
			bag.merge(element.visit(visitor));
		}
		
		return bag;
	}
}
