package dev.peerat.parser.java;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.function.Function;
import java.util.function.Predicate;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaFile implements JavaElement, ClassContainer, AnnotableBuffer{
	
	private List<Annotation> annotationBuffer;
	
	private Token pack;
	private List<Import> imports;
	private ClassBase mainClazz;
	private List<ClassBase> subClazz;
	
	public JavaFile(){
		this.imports = new ArrayList<>();
		this.subClazz = new ArrayList<>();
	}
	
	public void setPackage(Token token){
		this.pack = token;
	}
	
	public void addImport(Import value){
		this.imports.add(value);
	}
	
	@Override
	public void addClass(ClassBase clazz){
		if(mainClazz == null) this.mainClazz = clazz;
		else subClazz.add(clazz);
	}
	
	public Token getPackage(){
		return this.pack;
	}
	
	public List<Import> getImports(){
		return this.imports;
	}
	
	public ClassBase getMainClass(){
		return this.mainClazz;
	}
	
	public List<ClassBase> getSubClasses(){
		return this.subClazz;
	}
	
	@Override
	public void addAnnotationBuffer(Annotation annotation){
		if(annotationBuffer == null) annotationBuffer = new ArrayList<>();
		annotationBuffer.add(annotation);
	}
	
	public List<Annotation> getAnnotationBuffer(){
		List<Annotation> list = this.annotationBuffer;
		this.annotationBuffer = null;
		return list;
	}
	
	@Override
	public <E extends JavaElement> E find(Predicate<JavaElement> finder) {
		if(finder.test(mainClazz)) return (E) mainClazz;
		for(ClassBase clazz : subClazz){
			if(finder.test(clazz)) return (E) clazz;
		}
		return null;
	}
	
	@Override
	public <E extends JavaElement> void findAll(Predicate<JavaElement> finder, Collection<E> collector) {
		if(finder.test(mainClazz)) collector.add((E) mainClazz);
		mainClazz.findAll(finder, collector);
		for(ClassBase clazz : subClazz){
			if(finder.test(clazz)) collector.add((E) clazz);
			clazz.findAll(finder, collector);
		}
	}
	
	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor){
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		if(!visitor.canPropagate()) return bag;
		
		if(this.imports != null)
			for(Import imp : this.imports){
				bag.merge(imp.visit(visitor));
			}
		
		bag.merge(mainClazz.visit(visitor));
		
		if(this.subClazz != null)
			for(ClassBase clazz : this.subClazz){
				bag.merge(clazz.visit(visitor));
			}
		
		return bag;
	}
	
}
