package dev.peerat.parser.java.builder;

import java.util.ArrayList;
import java.util.List;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.ClassBase;
import dev.peerat.parser.java.Import;
import dev.peerat.parser.java.JavaFile;

public class JavaFileBuilder extends JavaBuilder<JavaFile>{
	
	private String pack;
	private List<Import> imports;
	private List<ClassBase> classes;
	
	public JavaFileBuilder setPackage(String value){
		this.pack = value;
		return this;
	}
	
	public JavaFileBuilder doImport(String value, boolean isStatic){
		if(this.imports == null) this.imports = new ArrayList<>();
		this.imports.add(new Import(isStatic, new Token(0,0, value, null)));
		return this;
	}
	
	public <T extends ClassBase> JavaFileBuilder clazz(JavaBuilder<T> builder){
		if(this.classes == null) this.classes = new ArrayList<>();
		this.classes.add(builder.build());
		return this;
	}
	
	public JavaFileBuilder normalize(JavaFile file){
		if(file.getPackage() != null) this.pack = file.getPackage().getValue();
		List<Import> imports = file.getImports();
		if(imports != null && imports.size() > 0){
			if(this.imports == null) this.imports = new ArrayList<>();
			for(Import imp : imports) this.imports.add(JavaBuilder.ofImport().normalize(imp).build());
		}
		//classes
		return this;
	}
	

	@Override
	public JavaFile build(){
		JavaFile result = new JavaFile();
		result.setPackage(new Token(0,0, this.pack, null));
		if(this.imports != null)
			for(Import imp : this.imports){
				result.addImport(imp);
			}
		if(this.classes != null)
			for(ClassBase clazz : this.classes){
				result.addClass(clazz);
			}
		return result;
	}
}

