package dev.peerat.parser.java.operation;

import java.util.Collection;
import java.util.function.Predicate;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class AssignOperation extends Operation{
	
	private Value left;
	private Value right;
	
	public AssignOperation(Value left, Value right){
		this.left = left;
		this.right = right;
	}
	
	public Value left(){
		return left;
	}
	
	public Value right(){
		return right;
	}
	
	@Override
	public <E extends JavaElement> E find(Predicate<JavaElement> finder) {
		return finder.test(left) ? (E)left : finder.test(right) ? (E)right : null;
	}
	
	@Override
	public <E extends JavaElement> void findAll(Predicate<JavaElement> finder, Collection<E> collector) {
		if(finder.test(left)) collector.add((E) left);
		left.findAll(finder, collector);
		if(finder.test(right)) collector.add((E) right);
		right.findAll(finder, collector);
	}

	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		bag.merge(left.visit(visitor));
		bag.merge(right.visit(visitor));
		return bag;
	}
}
