package dev.peerat.parser.java.operation;

import java.util.Collection;
import java.util.List;
import java.util.function.Function;
import java.util.function.Predicate;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class IfOperation extends OperationBag{
	
	private Value condition;
	
	public IfOperation(Value condition){
		super();
		this.condition = condition;
	}
	
	public Value getCondition(){
		return this.condition;
	}
	
	@Override
	public <E extends JavaElement> E find(Predicate<JavaElement> finder) {
		if(finder.test(condition)) return (E)condition;
		return super.find(finder);
	}

	@Override
	public <E extends JavaElement> void findAll(Predicate<JavaElement> finder, Collection<E> collector) {
		if(finder.test(condition)) collector.add((E)condition);
		condition.findAll(finder, collector);
		super.findAll(finder, collector);
	}
	
	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		bag.merge(condition.visit(visitor));
		bag.merge(super.visit(visitor));
		return bag;
	}
}
