package dev.peerat.parser.java.operation;

import java.util.Collection;
import java.util.List;
import java.util.function.Function;
import java.util.function.Predicate;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class SynchronizedOperation extends OperationBag{
	
	private Value value;
	
	public SynchronizedOperation(Value value){
		super();
		this.value = value;
	}
	
	public Value getValue(){
		return this.value;
	}
	
	@Override
	public <E extends JavaElement> E find(Predicate<JavaElement> finder) {
		if(finder.test(value)) return (E)value;
		return super.find(finder);
	}

	@Override
	public <E extends JavaElement> void findAll(Predicate<JavaElement> finder, Collection<E> collector) {
		if(finder.test(value)) collector.add((E)value);
		value.findAll(finder, collector);
		super.findAll(finder, collector);
	}
	
	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		bag.merge(this.value.visit(visitor));
		bag.merge(super.visit(visitor));
		return bag;
	}
}
