package dev.peerat.parser.java.operation;

import java.util.Collection;
import java.util.List;
import java.util.function.Function;
import java.util.function.Predicate;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class TryOperation extends OperationBag{

	private Variable resource;
	
	public TryOperation(){
		super();
	}
	
	public TryOperation(Variable resource){
		this();
		this.resource = resource;
	}
	
	public Variable getResource(){
		return this.resource;
	}
	
	@Override
	public <E extends JavaElement> E find(Predicate<JavaElement> finder) {
		return resource != null ? finder.test(resource) ? (E)resource : super.find(finder) : super.find(finder);
	}

	@Override
	public <E extends JavaElement> void findAll(Predicate<JavaElement> finder, Collection<E> collector) {
		if(resource != null){
			if(finder.test(resource)) collector.add((E) resource);
			resource.findAll(finder, collector);
		}
		super.findAll(finder, collector);
	}
	
	@Override
	public VisitorBag visit(Visitor<JavaElement> visitor) {
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		if(this.resource != null) bag.merge(this.resource.visit(visitor));
		bag.merge(super.visit(visitor));
		return bag;
	}
}
