package dev.peerat.parser.java.visitor;

import java.util.function.Predicate;

import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Class;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaClassVisitor extends JavaModifiableVisitor<dev.peerat.parser.java.Class, JavaClassVisitor>{

	private Predicate<String> nameFilter;
	private Visitor<JavaElement> elementVisitor;
	
	public JavaClassVisitor(boolean propagate){
		super(dev.peerat.parser.java.Class.class, propagate);
	}
	
	public JavaClassVisitor nameFilter(Predicate<String> validator){
		this.nameFilter = (this.nameFilter == null) ? validator : this.nameFilter.and(validator);
		return this;
	}
	
	public JavaClassVisitor elements(Visitor<JavaElement> visitor){
		this.elementVisitor = visitor;
		return this;
	}
	
	//				??
	//isSynthetic
	//isMandated
	//				??

	@Override
	public VisitorBag visitElement(Class element){
		VisitorBag bag = new VisitorBag();
		if(nameFilter != null){
			if(!nameFilter.test(element.getName().getValue())) return bag;
		}
		
		if(this.annotationVisitor != null){
			boolean validatedOne = false;
			for(Annotation annotation : element.getAnnotations()){
				VisitorBag visited = annotation.visit(this.annotationVisitor);
				if(visited.isValidated()){
					bag.merge(visited);
					validatedOne = true;
				}
			}
			if(!validatedOne) return bag;
		}
		
		if(this.modifierFilter != null){
			if(!this.modifierFilter.test(element.getModifier())) return bag;
		}
		
		if(this.elementVisitor != null){
			boolean validatedOne = false;
			for(JavaElement content : element.getElements()){
				VisitorBag visited = content.visit(this.elementVisitor);
				if(visited.isValidated()){
					bag.merge(visited);
					validatedOne = true;
				}
			}
			if(!validatedOne) return bag;
		}
		
		bag.compute(element);
		return bag;
	}

}
