package dev.peerat.parser.java.visitor;

import java.util.function.Predicate;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.JavaFile;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaFileVisitor extends JavaVisitor<JavaFile>{

	private boolean checkPackage;
	private boolean checkNoPackage;
	private Predicate<String> pack;
	private Visitor<JavaElement> imports;
	private Visitor<JavaElement> classes;
	
	public JavaFileVisitor(boolean propagate){
		super(JavaFile.class, propagate);
	}
	
	public JavaFileVisitor hasPackage(){
		this.checkPackage = true;
		this.checkNoPackage = false;
		return this;
	}
	
	public JavaFileVisitor hasNoPackage(){
		this.checkPackage = false;
		this.checkNoPackage = true;
		return this;
	}
	
	public JavaFileVisitor packaged(Predicate<String> predicate){
		this.pack = predicate;
		return this;
	}
	
	public JavaFileVisitor imp(Visitor<JavaElement> visitor){
		this.imports = visitor;
		return this;
	}
	
	public JavaFileVisitor clazz(Visitor<JavaElement> visitor){
		this.classes = visitor;
		return this;
	}

	@Override
	public VisitorBag visitElement(JavaFile element){
		VisitorBag bag = new VisitorBag();
		
		if(this.checkPackage && element.getPackage() == null) return bag;
		else if(this.checkNoPackage && element.getPackage() != null) return bag;
		
		if(this.pack != null && element.getPackage() != null && !this.pack.test(element.getPackage().getValue())) return bag;
		
		if(this.imports != null && validates(bag, element.getImports(), this.imports)) return bag;
		if(this.classes != null){
			VisitorBag resultMain = this.classes.visit(element.getMainClass());
			if(resultMain.isValidated()) bag.merge(resultMain);
			if(!(validates(bag, element.getSubClasses(), this.classes) || resultMain.isValidated())) return bag;
		}
		
		bag.compute(element);
		return bag;
	}
	
	

}
