package dev.peerat.parser.java.visitor.value;

import dev.peerat.parser.java.value.ArrayAccessValue;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.Operator;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaArrayAccessValueVisitor extends JavaVisitor<ArrayAccessValue>{

	private Visitor<Value> basePredicate;
	private Visitor<Value> accessPredicate;
	
	public JavaArrayAccessValueVisitor(boolean propagate){
		super(ArrayAccessValue.class, propagate);
	}
	
	public JavaArrayAccessValueVisitor checkBase(Visitor<Value> visitor){
		this.basePredicate = this.basePredicate == null ? visitor : Operator.and(this.basePredicate, visitor);
		return this;
	}
	
	public JavaArrayAccessValueVisitor accessor(Visitor<Value> visitor){
		this.accessPredicate = this.accessPredicate == null ? visitor : Operator.and(this.accessPredicate, visitor);
		return this;
	}

	@Override
	public VisitorBag visitElement(ArrayAccessValue element){
		VisitorBag bag = new VisitorBag();
		
		if(this.basePredicate != null){
			VisitorBag visited = this.basePredicate.visit(element.base());
			bag.merge(visited);
			if(!visited.isValidated()) return bag;
		}
		
		if(this.accessPredicate != null){
			VisitorBag visited = this.accessPredicate.visit(element.getAccessor());
			bag.merge(visited);
			if(!visited.isValidated()) return bag;
		}
		
		bag.compute(element);
		return bag;
	}
	
	

}
