package dev.peerat.parser.state;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.function.Predicate;

import dev.peerat.parser.Bag;
import dev.peerat.parser.TokenValidator;

public class BuilderStateTree<E, B> extends StateTree<B>{
	
	private BiFunction<E, Bag, B> builder;
	
	private List<StateTree<E>> childs;
	private List<StateTree<B>> multiple;
	private List<StateTree<B>> unique;

	public BuilderStateTree(BiFunction<E, Bag, B> builder){
		super();
		this.builder = builder;
		
		this.childs = new ArrayList<>();
		this.multiple = new ArrayList<>();
		this.unique = new ArrayList<>();
	}
	
	B build(TokenValidator validator, E element){
		B build = this.builder.apply(element, validator.getBag());
		validator.setBag(new Bag());
		
		BuilderStateTree<B, ?> builded = super.internalSeed(validator, build);
		if(builded != null) builded.build(validator, build);
		
		for(StateTree<B> state : this.multiple){
			TokenValidator branch;
			if(state.checker == null){
				while((builded = state.internalSeed((branch = validator.branch()), build)) != null){
					validator.merge(branch);
					builded.build(validator, build);
				}
			}else{
				while(state.checker.test((branch = validator.branch()))){
					builded = state.internalSeed(branch, build);
					if(builded == null) break;
					validator.merge(branch);
					builded.build(validator, build);
				}
			}
		}
		
		for(StateTree<B> state : this.unique){
			TokenValidator branch = validator.branch();
			if(state.checker == null){
				if((builded = state.internalSeed(branch, build)) != null){
					validator.merge(branch);
					builded.build(validator, build);
				}
			}else{
				if(state.checker.test(branch)){
					builded = state.internalSeed(branch, build);
					if(builded == null) break;
					validator.merge(branch);
					builded.build(validator, build);
				}
			}
		}
		
		BuilderStateTree<E, ?> sub_builded;
		for(StateTree<E> child : this.childs){
			TokenValidator branch = validator.branch();
			if(child.checker == null){
				sub_builded = child.internalSeed(branch, element);
				if(sub_builded == null) continue;
				validator.merge(branch);
				sub_builded.build(validator, element);
				break;
			}
			if(child.checker.test(branch)){
				sub_builded = child.internalSeed(branch, element);
				if(sub_builded == null) continue;
				validator.merge(branch);
				sub_builded.build(validator, element);
				break;
			}
		}
		
		return build;
	}
	
	public <T extends StateTree<E>> StateTree<E> thenNoChild(StateTree<E> child){
		childs.add(child);
		return child;
	}
	
	public <T extends StateTree<E>> StateTree<E> thenNoChild(Predicate<TokenValidator> checker){
		StateTree<E> state = new StateTree<>();
		state.checker = checker;
		this.childs.add(state);
		return state;
	}
	
	public <T extends BuilderStateTree<E,B>> BuilderStateTree<E,B> multiple(StateTree<B> child){
		this.multiple.add(child);
		return this;
	}
	
	public <T extends BuilderStateTree<E,B>> BuilderStateTree<E,B> multiple(Predicate<TokenValidator> checker){
		StateTree<B> state = new StateTree<>();
		state.checker = checker;
		return multiple(state);
	}
	
	public <T extends BuilderStateTree<E,B>> BuilderStateTree<E,B> unique(StateTree<B> child){
		this.unique.add(child);
		return this;
	}

	public <T extends StateTree<B>> StateTree<B> unique(Predicate<TokenValidator> checker){
		StateTree<B> state = new StateTree<>();
		state.checker = checker;
		unique(state);
		return state;
	}
}
