package dev.peerat.parser.state.generator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class GraphGenerator {
	
	public static void main(String[] args) throws Exception{
		File file = new File("java.generator");
		BufferedReader reader = new BufferedReader(new FileReader(file));
		List<String> lines = new ArrayList<>();
		String line;
		while((line = reader.readLine()) != null) lines.add(line);
		reader.close();
		
		Graph graph = new Graph(lines);
		graph.build();
	}
	
	public static class Graph{
		
		private static Pattern PATTERN = Pattern.compile("^(\\d+)\\s+->\\s+(.*)$");
		
		private List<Node> nodes;
		
		public Graph(List<String> lines){
			this.nodes = new ArrayList<>();
			for(String line : lines){
				Matcher matcher = PATTERN.matcher(line);
				if(matcher.matches()){
					Node from = nodes.get(Integer.parseInt(matcher.group(1)));
					String[] split = matcher.group(2).split("\\s+");
					for(String s : split){
						Node to = this.nodes.get(Integer.parseInt(s));
						from.addOutput(to);
						to.addInput(from);
					}
				}else{
					this.nodes.add(new Node(line));
				}
			}
		}
		
		public void build(){
			Set<Node> dones = new HashSet<>();
			Map<Node, Builder> builders = new HashMap<>();
			
			for(Node node : this.nodes){
				if(node.input.size() == 1 && node.input.iterator().next().output.size() == 1) continue;
				Builder builder = new Builder();
				builder.stack(node);
				builders.put(node, builder);
			}
			for(Node node : this.nodes){
				Node before;
				if(
						node.
						input.
						size() == 1 && (before = node.
						input.
						iterator().
						next()).
						output.
						size() == 1) {
					System.out.println(before.content+" ?");
					builders.
				get(before).
				stack(node);}
			}
			Set<Builder> stack = new HashSet<>(builders.values());
			while(!stack.isEmpty()){
				Builder builder = null;
				for(Builder b : stack){
					if(b.canBeBuild(dones)){
						builder = b;
						break;
					}
				}
				if(builder == null) throw new IllegalArgumentException("Illegale State");
				builder.build();
				dones.addAll(builder.stacks);
				stack.remove(builder);
			}
			
		}
		
		public static class Node{
			
			private String content;
			private Set<Node> input;
			private Set<Node> output;
			
			public Node(String content){
				this.content = content;
				this.input = new HashSet<>();
				this.output = new HashSet<>();
			}
			
			public void addInput(Node node){
				this.input.add(node);
			}
			
			public void addOutput(Node node){
				this.output.add(node);
			}
			
			@Override
			public String toString(){
				return ".validate("+content+")";
			}
		}
		
		public static class Builder{
			
			private List<Node> stacks;
			
			public Builder(){
				this.stacks = new ArrayList<>();
			}
			
			public void stack(Node node){
				this.stacks.add(node);
			}
			
			public boolean canBeBuild(Set<Node> dones){
				System.out.println("can ? "+dones+" -> "+stacks.get(0).input);
				return dones.containsAll(stacks.get(0).input);
			}
			
			public void build(){
				System.out.print("StateTree<?> st = new StateTree<>()");
				for(Node node : stacks) System.out.print(node);
				System.out.println(";");
			}
		}
		
	}
}
