package dev.peerat.parser.visitor;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.function.Predicate;

public class Operator{
	
	//Success ? Fail ?
	@SafeVarargs
	public static <E> Visitor<E> and(Visitor<E>... visitors){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				System.out.println("and "+Arrays.toString(visitors)+" -> "+element);
				VisitorBag bag = new VisitorBag();
				Set<Object> set = new HashSet<>();
				for(Visitor<E> visitor : visitors){
					if(!visitor.canVisit(element.getClass())) continue;
					VisitorBag vbag = visitor.visit(element);
					if(!vbag.isValidated()) return new VisitorBag();
					bag.merge(vbag);
					set.add(vbag.computed());
				}
				for(Object obj : set) bag.compute(obj);
				return bag;
			}

			@Override
			public boolean canVisit(Class<?> type){
				for(Visitor<E> visitor : visitors){
					if(visitor.canVisit(type)) return true;
				}
				return false;
			}
			
			@Override
			public boolean canPropagate(){
				return true;
			}
		};
	}
	
	//Quid de si c'est empty ?
	//Quid de si on a un collect dans les 2 ? ou dans 1 qui sourtout ne sera peux être pas appeler ?
	@SafeVarargs
	public static <E> Visitor<E> or(Visitor<E>... visitors){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag bag = new VisitorBag();
				for(Visitor<E> visitor : visitors){
					VisitorBag vbag = visitor.visit(element);
					if(!vbag.isValidated()) return new VisitorBag();
					bag.merge(vbag);
					bag.compute(vbag.computed());
					return bag;
				}
				return bag;
			}

			@Override
			public boolean canVisit(Class<?> type) {
				for(Visitor<E> visitor : visitors){
					if(visitor.canVisit(type)) return true;
				}
				return false;
			}
			
			@Override
			public boolean canPropagate(){
				return true;
			}
		};
	}
	
	public static <E> Visitor<E> not(Visitor<E> visitor){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag result = visitor.visit(element);
				if(result.isValidated()) return null;
				VisitorBag bag = new VisitorBag();
				bag.compute(element);
				return bag;
			}
			
			@Override
			public boolean canVisit(Class<?> type){
				return visitor.canVisit(type);
			}
			
			@Override
			public boolean canPropagate(){
				return visitor.canPropagate();
			}
		};
	}
	
	public static <E> Visitor<E> key(Visitor<E> visitor){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				return visitor.visit(element).pair(element);
			}

			@Override
			public boolean canVisit(Class<?> type){
				return visitor.canVisit(type);
			}
			
			@Override
			public boolean canPropagate(){
				return visitor.canPropagate();
			}
		};
	}
	
	public static <E> Visitor<E> value(Visitor<E> visitor){
		return collect(visitor);
	}
	
	public static <E> Visitor<E> collect(Visitor<E> visitor){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				return visitor.visit(element).collect();
			}

			@Override
			public boolean canVisit(Class<?> type){
//				System.out.println("collect can visit ? "+type);
				return visitor.canVisit(type);
			}
			
			@Override
			public boolean canPropagate(){
				return visitor.canPropagate();
			}
		};
	}
	
	public static <E> Visitor<E> count(Visitor<E> visitor, Predicate<Integer> validator){
		return new Visitor<E>(){
			@Override
			public VisitorBag visit(E element){
				VisitorBag visited = visitor.visit(element);
				return visited.isValidated() && validator.test(visited.toList().size()) ? visited : new VisitorBag();
			}

			@Override
			public boolean canVisit(Class<?> type){
				return visitor.canVisit(type);
			}
			
			@Override
			public boolean canPropagate(){
				return visitor.canPropagate();
			}
		};
	}
	
}
