package dev.peerat.parser.visitor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//only one pass in parameter or visit ?
public class VisitorBag{
	
	private static final Object VALIDATED_BAG = new Object();
	
	private List<Object> collect;
	private Object computed;
	
	public VisitorBag(){
		this.collect = new ArrayList<>(0);
	}
	
	public void compute(Object element){
		this.computed = element;
	}
	
	public void merge(VisitorBag bag){
		this.collect.addAll(bag.collect);
		if((!this.collect.isEmpty()) && this.computed == null) this.computed = VALIDATED_BAG; 
	}
	
	public Object computed(){
		return this.computed;
	}
	
	public boolean isValidated(){
		return this.computed != null;
	}
	
	VisitorBag pair(Object key){
		this.collect = Arrays.asList(new Pair<>(key, this.collect));
		return this;
	}
	
	VisitorBag collect(){
		if(this.computed != null) this.collect.add(this.computed);
		return this;
	}
	
	public boolean hasElement(){
		return !this.collect.isEmpty();
	}
	
	public <T> T toElement(){
		return (T) this.collect.get(0);
	}
	
	public <T> List<T> toList(){
		return (List<T>) this.collect;
	}
	
	public <K, V> Map<K, V> toMap(){
		Map<K, V> map = new HashMap<>();
		for(Object obj : this.collect){
			Pair<K, V> pair = (Pair<K, V>)obj;
			map.put(pair.getKey(), pair.getValue());
		}
		return map;
	}
	
	public <K, V> Map<K, List<V>> toMapValues(){
		Map<K, List<V>> map = new HashMap<>();
		for(Object obj : this.collect){
			Pair<K, V> pair = (Pair<K, V>)obj;
			map.put(pair.getKey(), pair.getValues());
		}
		return map;
	}
	
	public static class Pair<K, V>{
		
		private Object key;
		private List<Object> values;
		
		public Pair(Object key, List<Object> values){
			this.key = key;
			this.values = values;
		}
		
		public K getKey(){
			return (K) this.key;
		}
		
		public V getValue(){
			return (V) this.values.get(0);
		}
		
		public List<V> getValues(){
			return (List<V>) this.values;
		}
		
		@Override
		public String toString(){
			return "[key="+key+", values="+values+"]";
		}
	}
}